#define F_CPU 16000000
#include <util/delay.h>
#include <avr/io.h>

#define SUS 0x04
#define JOS 0x01
#define DREAPTA 0x02
#define STANGA 0x08
#define DIRSUS 1
#define DIRJOS 3
#define DIRDREAPTA 2
#define DIRSTANGA 4
#define MAXLEN 16
#define INITLEN 4

int line_a[8]={16,0,0,2,0,4,64,128};
int line_c[8]={0,32,16,0,4,0,0,0};
int column_a[8]={0,0,0,0,8,0,32,1};
int column_c[8]={8,64,2,128,0,1,0,0};

typedef struct{
	int x,y;
}point;

point snake[MAXLEN]; //informatii snake
int len,dir,reset_flag,seed,on_food;
point food;

void init(){//intializez si resetez starea jocului
	int i;
	len=INITLEN;
	reset_flag=0;
	on_food=1;
	dir=DIRSTANGA;
	for (i=0;i<INITLEN;i++){
		snake[i].x=4;
		snake[i].y=i+2;
	}
}

void print_pixel(int i,int j){//aprinderea oricarui led
	int l;
	PORTC=0x00;
	PORTA=0x00;
	for (l=0;l<8;l++){
		if (j==l){
			if (column_a[j]!=0)
				PORTA |= column_a[j];
			if (column_c[j]!=0)
				PORTC |= column_c[j];
		}
		if (i!=l){
			if (line_a[l]!=0)
				PORTA |= line_a[l];
			if (line_c[l]!=0)
				PORTC |= line_c[l];
		}
	}
	_delay_ms(3);
}

point rand_point(int seed){//generaree numere pseudo random
	point tmp;
	tmp.x=(seed*109+89)%8;
	tmp.y=((seed+37)*109+89)%8;
	return tmp;
}

void setare_matrice(){//setarea porturilor de iesire
	DDRA = 0xFF;
	DDRC = 0xFF;
}

void setare_butoane() {
	DDRB = 0x00;	
	PORTB = 0x0F;	//activez rezistentele de pull-up
} 

void adjust(){//ajustarea valorilor la trecerea prin perete
	int i;
	for (i=0;i<len;i++){
		if (snake[i].x<0)
			snake[i].x=7;
		if (snake[i].x>7)
			snake[i].x=0;
		if (snake[i].y<0)
			snake[i].y=7;
		if (snake[i].y>7)
			snake[i].y=0;
	}
}

void print_snake(){//printare snake
	int i;
	adjust();
	for (i=0;i<len;i++){
		print_pixel(snake[i].x,snake[i].y);
	}
}

int ciocnire(){//verificarea ciocnire cu peretele
	int i;
	for (i=1;i<len;i++)
		if (snake[0].x==snake[i].x && snake[0].y==snake[i].y)
			return 1;
	return 0;
}

void get_food(){//gasire valoare unde sa puna "marul"
	int i,adev;
	while (1){
		food=rand_point(seed);
		adev=0;
		for (i=0;i<len;i++)
			if (food.x==snake[i].x && food.y==snake[i].y)
				adev=1;
		if (adev==0)
			break;
		seed++;
	}
}

void move_snake(int directie){//miscare sarpe
	int i;
	for (i=len-1;i>0;i--)
		snake[i]=snake[i-1];
	if (directie==DIRSUS){
		snake[0].x=snake[1].x-1;
		snake[0].y=snake[1].y;
	}
	if (directie==DIRJOS){
		snake[0].x=snake[1].x+1;
		snake[0].y=snake[1].y;
	}
	if (directie==DIRSTANGA){
		snake[0].x=snake[1].x;
		snake[0].y=snake[1].y-1;
	}
	if (directie==DIRDREAPTA){
		snake[0].x=snake[1].x;
		snake[0].y=snake[1].y+1;
	}
}

int main(){
	int i = 0,j=0;
	seed=17;
	setare_matrice();
	setare_butoane();
	init();
	while (14){
		j=dir;//noua directie de mers
		if (!(PINB & SUS))
			dir=DIRSUS;
		else if (!(PINB & JOS))
			dir=DIRJOS;
		else if (!(PINB & STANGA))
			dir=DIRSTANGA;
		else if (!(PINB & DREAPTA))
			dir=DIRDREAPTA;
		if (dir-j == -2 || dir-j == 2)
			dir=j;
		move_snake(dir);
		if (on_food==1){//printare "mar"
			get_food();
			on_food=0;
			seed++;
		}
		for (i=0;i<18-len;i++){//printare
			print_snake();
			print_pixel(food.x,food.y);
		}
		if (ciocnire()==1){//ciocnire +resetarw
			for (i=0;i<8;i++)
				for (j=0;j<8;j++){
					print_pixel(i,j);
					_delay_ms(10);
				}
			init();
			
		}//marire lungime snake
		if (snake[len-1].x==food.x && snake[len-1].y==food.y){
			on_food=1;
			len++;
			if (len>MAXLEN){
				len=MAXLEN;
			}
			else{
				snake[len-1].x=food.x;
				snake[len-1].y=food.y;
			}
		}
	}
	return 0;
}